/*
 * Copyright (c) 2004 Hamish Rodda <rodda@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "attributes.h"

#include <qcheckbox.h>

#include <kdebug.h>
#include <knuminput.h>
#include <kconfig.h>

#include "settingsbridge.h"

#include <X11/Xlib.h>
#include <X11/extensions/NVCtrl.h>

extern "C" {
	#include <X11/extensions/NVCtrlLib.h>
}

AbstractAttribute::AbstractAttribute( SettingsBridge * bridge, QWidget * widget )
	: m_bridge(bridge)
	, m_widget(widget)
	, m_valid(false)
{
}

bool AbstractAttribute::isValid() const
{
	return m_valid;
}

SettingsBridge * AbstractAttribute::bridge( ) const
{
	return m_bridge;
}

QWidget * AbstractAttribute::widget( ) const
{
	return m_widget;
}

void AbstractAttribute::setValid( bool valid )
{
	m_valid = valid;
}

SimpleAttribute::SimpleAttribute( SettingsBridge * b, QWidget * widget, int attribute, int displayMask, bool reverse )
	: AbstractAttribute(b, widget)
	, m_attribute(attribute)
	, m_displayMask(displayMask)
	, m_reverse(reverse)
	, m_changed(false)
	, m_appliesImmediately(false)
{
	setValid(XNVCTRLQueryValidAttributeValues(qt_xdisplay(), bridge()->currentScreen(), m_displayMask, m_attribute, &validValues));
	
	if (!(validValues.permissions & ATTRIBUTE_TYPE_WRITE))
		this->widget()->setEnabled(false);
	
	switch (validValues.type) {
		case ATTRIBUTE_TYPE_INTEGER:
		case ATTRIBUTE_TYPE_BITMASK:
		case ATTRIBUTE_TYPE_BOOL:
			break;
			
		case ATTRIBUTE_TYPE_RANGE:
			if (this->widget()->inherits("KIntNumInput"))
				static_cast<KIntNumInput*>(this->widget())->setRange(validValues.u.range.min, validValues.u.range.max);
			break;
			
		case ATTRIBUTE_TYPE_INT_BITS:
			break;
	}
}

int SimpleAttribute::value(int screen) const
{
	return bridge()->queryAttribute(attribute(), displayMask(), screen);
}

void SimpleAttribute::update( )
{
	if (widget()->inherits("QCheckBox"))
		static_cast<QCheckBox*>(widget())->setChecked(value() xor m_reverse);
	
	else if (widget()->inherits("QComboBox"))
		static_cast<QComboBox*>(widget())->setCurrentItem(value());
		
	else if (widget()->inherits("KIntNumInput"))
		static_cast<KIntNumInput*>(widget())->setValue(value());
		
	else
		Q_ASSERT(false);
		
	if (isChanged())
		checkChanged();
}

void SimpleAttribute::makeConnections()
{
	if (widget()->inherits("QCheckBox"))
		connect(widget(), SIGNAL(toggled(bool)), SLOT(setAttribute(bool)));
	
	else if (widget()->inherits("QComboBox"))
		connect(widget(), SIGNAL(activated(int)), SLOT(setAttribute(int)));
		
	else if (widget()->inherits("KIntNumInput"))
		connect(widget(), SIGNAL(valueChanged(int)), SLOT(setAttribute(int)));
		
	else
		Q_ASSERT(false);
}

void SimpleAttribute::setAttribute( int v )
{
	setAttribute(-1, attribute(), v);
}

void SimpleAttribute::setAttribute( bool v )
{
	setAttribute(-1, attribute(), v xor m_reverse);
}

void SimpleAttribute::checkChanged( )
{
	bool c = false;
	for (QMap<int,int>::ConstIterator it = m_proposedValues.begin(); it != m_proposedValues.end(); ++it)
		if (value(it.key()) != it.data()) {
			c = true;
			break;
		}
	
	if (m_changed != c) {
		m_changed = c;
		emit changed(m_changed);
	}
}

int SimpleAttribute::attribute( ) const
{
	return m_attribute;
}

int SimpleAttribute::proposedValue(int screen) const
{
	if (m_proposedValues.contains(screen == -1 ? bridge()->currentScreen() : screen))
		return m_proposedValues[screen == -1 ? bridge()->currentScreen() : screen];
		
	Q_ASSERT(false);
	return -1;
}

bool SimpleAttribute::isChanged() const
{
	return m_changed;
}

void SimpleAttribute::commitChange( )
{
	for (QMap<int,int>::Iterator it = m_proposedValues.begin(); it != m_proposedValues.end(); ++it) {
		// FIXME worry about errors?
		bridge()->setAttribute(attribute(), proposedValue(), displayMask());
		m_proposedValues.remove(it);
	}
	
	m_changed = false;
}

void SimpleAttribute::revertChange( )
{
	m_proposedValues.clear();
	m_changed = false;
	
	update();
}

int SimpleAttribute::displayMask( ) const
{
	return m_displayMask;
}

bool SimpleAttribute::appliesImmediately( ) const
{
	return m_appliesImmediately;
}

void SimpleAttribute::setAppliesImmediately( bool immediate )
{
	if (m_appliesImmediately != immediate) {
		m_appliesImmediately = immediate;
		if (m_appliesImmediately && isChanged())
			commitChange();
	};
}

void SimpleAttribute::checkUpdate( int attr, int dMask )
{
	if (attr != attribute())// || dMask != displayMask())
		return;
	
	update();
}

bool SimpleAttribute::concernsAttribute( int a, int dm ) const
{
	return a == attribute() && dm == displayMask();
}

void SimpleAttribute::setAttribute( int screen, int a, int v )
{
	Q_ASSERT(a == attribute());
	if (a != attribute())
		return;
	
	m_proposedValues[screen == -1 ? bridge()->currentScreen() : screen] = v;
	if (!appliesImmediately())
		checkChanged();
	else
		commitChange();
}

void SimpleAttribute::save( KConfig * config, int screen ) const
{
	config->writeEntry(bridge()->attributeName(attribute(), displayMask(), screen), value());
}

CombinedAttribute::CombinedAttribute( SettingsBridge * bridge, QWidget * widget, int boolAttribute, int intAttribute )
	: AbstractAttribute(bridge, widget)
	, m_boolAttrib(new SimpleAttribute(bridge, widget, boolAttribute))
	, m_intAttrib(new  SimpleAttribute(bridge, widget, intAttribute))
{
	Q_ASSERT(this->widget()->inherits("QComboBox"));

	connect(m_boolAttrib, SIGNAL(changed(bool)), SIGNAL(changed(bool)));
	connect(m_intAttrib, SIGNAL(changed(bool)), SIGNAL(changed(bool)));
	
	setValid(m_boolAttrib->isValid() && m_intAttrib->isValid());
	
	if (m_intAttrib->validValues.type == ATTRIBUTE_TYPE_INT_BITS) {
		int j = 0;
		for (int i = 0; i < int(sizeof(int) * 8); ++i) {
			if (!(m_intAttrib->validValues.u.bits.ints >> i))
				break;
				
			if (!((1 << i) & m_intAttrib->validValues.u.bits.ints))
				this->widget()->removeItem(j + 1);
			else
				j++;
		}
	}
}

void CombinedAttribute::update( )
{
	int value1 = bridge()->queryAttribute(m_boolAttrib->attribute());
	int value2 = bridge()->queryAttribute(m_intAttrib->attribute());
	
	if (value1)
		widget()->setCurrentItem(0);
	
	else {
		// Calculate the actual index of this item
		int val = value2;
		if (m_intAttrib->validValues.type == ATTRIBUTE_TYPE_INT_BITS)
			for (int i = 0; i <= value2; ++i)
				if (!((1 << i) & m_intAttrib->validValues.u.bits.ints))
					val--;
		
 		widget()->setCurrentItem(val + 1);
	}
}

void CombinedAttribute::makeConnections( )
{
	if (widget()->inherits("QComboBox"))
		connect(widget(), SIGNAL(activated(int)), SLOT(setAttribute(int)));
		
	else
		Q_ASSERT(false);
}

void CombinedAttribute::setAttribute( int value )
{
	if (value == 0) {
		m_boolAttrib->setAttribute(true);
		
	} else {
		m_boolAttrib->setAttribute(false);
		
		// Now, to calculate the actual index of this item
		int val = value - 1;
		if (m_intAttrib->validValues.type == ATTRIBUTE_TYPE_INT_BITS)
			for (int i = 0; i <= val; ++i)
				if (!((1 << i) & m_intAttrib->validValues.u.bits.ints))
					val++;
		
		m_intAttrib->setAttribute(val);
	}
}

bool CombinedAttribute::isChanged() const
{
	return m_boolAttrib->isChanged() || m_intAttrib->isChanged();
}

void CombinedAttribute::commitChange()
{
	m_boolAttrib->commitChange();
	m_intAttrib->commitChange();
}

void CombinedAttribute::revertChange( )
{
	m_boolAttrib->revertChange();
	m_intAttrib->revertChange();
}

QComboBox* CombinedAttribute::widget( ) const
{
	return static_cast<QComboBox*>(AbstractAttribute::widget());
}

void CombinedAttribute::checkUpdate( int attribute, int displayMask )
{
	m_boolAttrib->checkUpdate(attribute, displayMask);
	m_intAttrib->checkUpdate(attribute, displayMask);
}

bool CombinedAttribute::concernsAttribute( int a, int dm ) const
{
	return m_intAttrib->concernsAttribute(a, dm) || m_boolAttrib->concernsAttribute(a, dm);
}

void CombinedAttribute::setAttribute( int screen, int a, int v )
{
	if (m_boolAttrib->attribute() == a)
		m_boolAttrib->setAttribute(screen, a, v);
		
	else if (m_intAttrib->attribute() == a)
		m_intAttrib->setAttribute(screen, a, v);
}

void CombinedAttribute::save( KConfig * config, int screen ) const
{
	m_boolAttrib->save(config, screen);
	m_intAttrib->save(config, screen);
}

#include "attributes.moc"
