/*
 * Copyright (c) 2004 Hamish Rodda <rodda@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "nvidiadisplaymodule.h"

#include <qcheckbox.h>
#include <qwhatsthis.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qtabwidget.h>

#include <kdebug.h>
#include <klocale.h>
#include <kcmodule.h>
#include <kglobal.h>
#include <kgenericfactory.h>
#include <kcombobox.h>
#include <kconfig.h>
#include <knuminput.h>

#include "nvidiadisplayswidget.h"
#include "nvidiacrtwidget.h"
#include "nvidiatvwidget.h"
#include "nvidiaflatpanelwidget.h"
#include "settingsbridge.h"

#include <X11/Xlib.h>
#include <X11/extensions/NVCtrl.h>

// DLL Interface for kcontrol
typedef KGenericFactory<NvidiaDisplayModule, QWidget > KSSFactory;
K_EXPORT_COMPONENT_FACTORY (kcm_nvidiadisplay, KSSFactory("nvidiadisplay") )

NvidiaDisplayModule::NvidiaDisplayModule( QWidget * parent, const char * name, const QStringList & _args )
	: NvidiaModule(parent, name, _args)
	, m_main(new NvidiaDisplaysWidget(this))
{
	if (!m_settings->isValid())
		return;

	/*for (uint i = 0; i <= (uint)QApplication::desktop()->numScreens(); i++) {
		if (m_settings->screenValid(i)) {
			// Set up screens here
		}
	}*/
	
	if (m_settings->validScreens().count() <= 1)
		m_main->screenContainer->hide();

	while (m_main->displayTabWidget->currentPage())
		m_main->displayTabWidget->removePage(m_main->displayTabWidget->currentPage());
	
	m_main->labelProcessor->setText(m_settings->queryString(NV_CTRL_STRING_PRODUCT_NAME));
	
	switch (m_settings->queryAttribute(NV_CTRL_BUS_TYPE)) {
		case NV_CTRL_BUS_TYPE_AGP:
			m_main->labelBus->setText(i18n("AGP"));
			break;
		
		case NV_CTRL_BUS_TYPE_PCI:
			m_main->labelBus->setText(i18n("PCI"));
			break;
		
		case NV_CTRL_BUS_TYPE_PCI_EXPRESS:
			m_main->labelBus->setText(i18n("PCI Express"));
			break;
		
		default:
			Q_ASSERT(false);
			m_main->labelBus->setText(i18n("Unknown"));
			break;
	}
	
	m_main->labelIRQ->setText(QString::number(m_settings->queryAttribute(NV_CTRL_IRQ)));
	m_main->labelVideoMemory->setText(i18n("%1 Kb").arg(KGlobal::locale()->formatNumber(m_settings->queryAttribute(NV_CTRL_VIDEO_RAM), 0)));
	
	QString os, arch;
	
	switch (m_settings->queryAttribute(NV_CTRL_OPERATING_SYSTEM)) {
		case NV_CTRL_OPERATING_SYSTEM_LINUX:
			os = i18n("Linux");
			break;
			
		case NV_CTRL_OPERATING_SYSTEM_FREEBSD:
			os = i18n("FreeBSD");
			break;
		
		default:
			m_main->labelOS->setText(i18n("Unknown Operating System"));
			Q_ASSERT(false);
			break;
	}
	
	switch (m_settings->queryAttribute(NV_CTRL_ARCHITECTURE)) {
		case NV_CTRL_ARCHITECTURE_X86:
			arch = i18n("x86");
			break;
		
		case NV_CTRL_ARCHITECTURE_X86_64:
			m_main->labelOS->setText(i18n("x86_64"));
			break;
	
		case NV_CTRL_ARCHITECTURE_IA64:
			m_main->labelOS->setText(i18n("ia64"));
			break;
			
		default:
			m_main->labelOS->setText(i18n("Unknown Architecture"));
			Q_ASSERT(false);
			break;
	}
	
	m_main->labelOS->setText(QString("%1 %2").arg(os).arg(arch));
	m_main->labelBios->setText(m_settings->queryString(NV_CTRL_STRING_VBIOS_VERSION));
	m_main->labelDriverVersion->setText(m_settings->queryString(NV_CTRL_STRING_NVIDIA_DRIVER_VERSION));
	
#define DISPLAY_DEVICE_CRT_MASK 0x000000FF
#define DISPLAY_DEVICE_TV_MASK  0x0000FF00
#define DISPLAY_DEVICE_DFP_MASK 0x00FF0000
	
	int displays = m_settings->queryAttribute(NV_CTRL_CONNECTED_DISPLAYS);
	int enabled = m_settings->queryAttribute(NV_CTRL_ENABLED_DISPLAYS);
	for (int i = 0; (1 << i) <= displays && i < int(sizeof(int) * 8); ++i) {
		if (!((1 << i) & displays)) continue;
		
		int currentDisplayMask = 1 << i;
		QWidget* w = 0L;
		if (currentDisplayMask & DISPLAY_DEVICE_CRT_MASK) {
			NvidiaCRTWidget* newWidget = new NvidiaCRTWidget(m_main);
			w = newWidget;
			m_settings->registerWidget(newWidget->displayDigitalVibrance, NV_CTRL_DIGITAL_VIBRANCE, currentDisplayMask, true);
			m_settings->registerWidget(newWidget->displayImageSharpening, NV_CTRL_IMAGE_SHARPENING, currentDisplayMask, true);
		
		} else if (currentDisplayMask & DISPLAY_DEVICE_DFP_MASK) {
			NvidiaFlatpanelWidget* newWidget = new NvidiaFlatpanelWidget(m_main);
			w = newWidget;
			m_settings->registerWidget(newWidget->displayDigitalVibrance, NV_CTRL_DIGITAL_VIBRANCE, currentDisplayMask, true);
			m_settings->registerWidget(newWidget->displayImageSharpening, NV_CTRL_IMAGE_SHARPENING, currentDisplayMask, true);
			m_settings->registerWidget(newWidget->displayScaling, NV_CTRL_FLATPANEL_SCALING, currentDisplayMask);
			m_settings->registerWidget(newWidget->displayDithering, NV_CTRL_FLATPANEL_DITHERING, currentDisplayMask);
			if (!newWidget->displayDithering->isVisible())
				newWidget->textLabel1_2->hide();
		
		} else if (currentDisplayMask & DISPLAY_DEVICE_TV_MASK) {
			NvidiaTVWidget* newWidget = new NvidiaTVWidget(m_main);
			w = newWidget;
			newWidget->textLabelEncoder->setText(m_settings->queryString(NV_CTRL_STRING_TV_ENCODER_NAME, currentDisplayMask));
			m_settings->registerWidget(newWidget->overscan, NV_CTRL_TV_OVERSCAN, currentDisplayMask, true);
			m_settings->registerWidget(newWidget->flickerFilter, NV_CTRL_TV_FLICKER_FILTER, currentDisplayMask, true);
			m_settings->registerWidget(newWidget->brightness, NV_CTRL_TV_BRIGHTNESS, currentDisplayMask, true);
			m_settings->registerWidget(newWidget->hue, NV_CTRL_TV_HUE, currentDisplayMask, true);
			m_settings->registerWidget(newWidget->contrast, NV_CTRL_TV_CONTRAST, currentDisplayMask, true);
			m_settings->registerWidget(newWidget->saturation, NV_CTRL_TV_SATURATION, currentDisplayMask, true);
			//NV_CTRL_TV_RESET_SETTINGS
		}
		
		Q_ASSERT(w);
		if (!(((1 << i) & displays) & enabled))
			w->setEnabled(false);
		
		m_main->displayTabWidget->addTab(w, m_settings->queryString(NV_CTRL_STRING_DISPLAY_DEVICE_NAME, currentDisplayMask));
	}
		
	m_settings->registerWidget(m_main->displayTwinview, NV_CTRL_TWINVIEW);
}

#include "nvidiadisplaymodule.moc"
